@@ -18,18 +18,12 @@ class GraphicsWidgets : public ControlsWidget
 {
 public:
   GraphicsWidgets(EventDisplayer *ed);
-  virtual ~GraphicsWidgets();
+  virtual ~GraphicsWidgets() {}
   virtual void populateSubMenu(Wt::WMenu *menu);
   
 private:
-  Wt::WSignalMapper<Wt::WColor > * colorMapper_;
-
   Wt::WWidget* emwebLogo();
   Wt::WWidget* paintbrush();
-  void addColor(Wt::WTable* table, 
-		int row, 
-		int column, 
-		const Wt::WColor& color);
 };
 
 #endif
