@@ -2,6 +2,7 @@
 
 #include <Wt/WPaintedWidget>
 #include <Wt/WPainterPath>
+#include <Wt/WColor>
 #include <Wt/WEvent>
 
 #include <vector>
@@ -28,13 +29,13 @@ protected:
   virtual void paintEvent(WPaintDevice *paintDevice);
 
 private:
-  bool dragging_;
   WPainterPath path_;
   WColor color_;
 
   void mouseDown(const WMouseEvent& e); 
-  void mouseUp(const WMouseEvent& e);
-  void drag(const WMouseEvent& e);
+  void mouseDrag(const WMouseEvent& e);
+  void touchStart(const WTouchEvent& e);
+  void touchMove(const WTouchEvent& e);
 };
 
 #endif // PAINTBRUSH_H_
