@@ -40,9 +40,9 @@ void InitSchema::actId(V& value, const std::string& name, int size)
   mapping_.naturalIdFieldSize = size;
 
   if (mapping_.surrogateIdFieldName)
-    throw std::logic_error("Error: Wt::Dbo::id() called for class C "
-			   "with surrogate key: "
-			   "Wt::Dbo::dbo_traits<C>::surrogateIdField() != 0");
+    throw Exception("Error: Wt::Dbo::id() called for class C "
+		    "with surrogate key: "
+		    "Wt::Dbo::dbo_traits<C>::surrogateIdField() != 0");
 
   idField_ = true;
   field(*this, value, name, size);
@@ -57,9 +57,9 @@ void InitSchema::actId(ptr<C>& value, const std::string& name, int size,
   mapping_.naturalIdFieldSize = size;
 
   if (mapping_.surrogateIdFieldName)
-    throw std::logic_error("Error: Wt::Dbo::id() called for class C "
-			   "with surrogate key: "
-			   "Wt::Dbo::dbo_traits<C>::surrogateIdField() != 0");
+    throw Exception("Error: Wt::Dbo::id() called for class C "
+		    "with surrogate key: "
+		    "Wt::Dbo::dbo_traits<C>::surrogateIdField() != 0");
 
   idField_ = true;
   actPtr(PtrRef<C>(value, name, size, fkConstraints));
@@ -91,25 +91,34 @@ void InitSchema::actPtr(const PtrRef<C>& field)
 {
   Session::Mapping<C> *mapping = session_.getMapping<C>();
 
-  foreignKeyName_ = field.name();
-  foreignKeyTable_ = mapping->tableName;
-  fkConstraints_ = field.fkConstraints();
+  bool setName = foreignKeyName_.empty();
+
+  if (setName) {
+    foreignKeyName_ = field.name();
+    foreignKeyTable_ = mapping->tableName;
+    fkConstraints_ = field.fkConstraints();
+  }
 
   field.visit(*this, &session_);
 
-  foreignKeyName_.clear();
-  foreignKeyTable_.clear();
-  fkConstraints_ = 0;
+  if (setName) {
+    foreignKeyName_.clear();
+    foreignKeyTable_.clear();
+    fkConstraints_ = 0;
+  }
 }
 
 template<class C>
 void InitSchema::actCollection(const CollectionRef<C>& field)
 {
   const char *joinTableName = session_.tableName<C>();
+  std::string joinName = field.joinName();
+  if (joinName.empty())
+    joinName = mapping_.tableName;
 
   mapping_.sets.push_back
-    (Session::SetInfo(joinTableName, field.type(), field.joinName(),
-		      field.joinId(), field.fkConstraints()));
+    (Session::SetInfo(joinTableName, field.type(), joinName, field.joinId(),
+		      field.fkConstraints()));
 }
 
     /*
@@ -121,7 +130,7 @@ void DropSchema::visit(C& obj)
 {
   persist<C>::apply(obj, *this);
 
-  drop(tableName_);
+  drop(mapping_.tableName);
 }
 
 template<typename V>
@@ -150,7 +159,9 @@ void DropSchema::actCollection(const CollectionRef<C>& field)
   } else {
     const char *tableName = session_.tableName<C>();
     if (tablesDropped_.count(tableName) == 0) {
-      DropSchema action(session_, tableName, tablesDropped_);
+      DropSchema action(session_, 
+			*session_.getMapping(tableName), 
+			tablesDropped_);
       C dummy;
       action.visit(dummy);
     }
@@ -166,7 +177,7 @@ void DboAction::actCollection(const CollectionRef<C>& field)
 {
   Session::SetInfo *setInfo = &mapping_->sets[setIdx_++];
 
-  if (dbo_->isPersisted()) {
+  if (dbo_->session()) {
     int statementIdx = Session::FirstSqlSelectSet + setStatementIdx_;
 
     const std::string& sql
@@ -189,13 +200,13 @@ void DboAction::actCollection(const CollectionRef<C>& field)
 template<typename V>
 void LoadBaseAction::act(const FieldRef<V>& field)
 {
-  field.setValue(*dbo().session(), statement_, column_++);
+  field.setValue(*session(), statement_, column_++);
 }
 
 template<class C>
 void LoadBaseAction::actPtr(const PtrRef<C>& field)
 {
-  field.visit(*this, dbo().session());
+  field.visit(*this, session());
 }
 
 template <class C>
@@ -273,7 +284,7 @@ void SaveBaseAction::actPtr(const PtrRef<C>& field)
     break;
   case Self:
     bindNull_ = !field.value();
-    field.visit(*this, dbo().session());
+    field.visit(*this, session());
     bindNull_ = false;
 
     break;
@@ -308,8 +319,7 @@ void SaveBaseAction::actCollection(const CollectionRef<C>& field)
 
 	SqlStatement *statement;
 
-	statement = dbo().session()->getStatement(mapping().tableName,
-						  statementIdx);
+	statement = session()->getStatement(mapping().tableName, statementIdx);
 	{
 	  ScopedStatementUse use(statement);
 
@@ -333,8 +343,7 @@ void SaveBaseAction::actCollection(const CollectionRef<C>& field)
 	// Sql delete
 	++statementIdx;
 
-	statement = dbo().session()->getStatement(mapping().tableName,
-						  statementIdx);
+	statement = session()->getStatement(mapping().tableName, statementIdx);
 
 	{
 	  ScopedStatementUse use(statement);
@@ -599,6 +608,9 @@ void SetReciproceAction::actCollection(const CollectionRef<C>& field)
 template<class C>
 void ToAnysAction::visit(const ptr<C>& obj)
 {
+  if (!session_ && obj.session())
+    session_ = obj.session();
+
   if (dbo_traits<C>::surrogateIdField())
     result_.push_back(obj.id());
 
@@ -678,15 +690,14 @@ void FromAnyAction::visit(const ptr<C>& obj)
 {
   if (dbo_traits<C>::surrogateIdField()) {
     if (index_ == 0)
-      throw std::logic_error("dbo_result_traits::setValues(): "
-			     "cannot set surrogate id.");
+      throw Exception("dbo_result_traits::setValues(): cannot set surrogate id.");
     --index_;
   }
 
   if (dbo_traits<C>::versionField()) {
     if (index_ == 0)
-      throw std::logic_error("dbo_result_traits::setValues(): "
-			     "cannot set version field.");
+      throw Exception("dbo_result_traits::setValues(): "
+		      "cannot set version field.");
     --index_;
   }
 
@@ -712,6 +723,19 @@ struct FromAny<Enum, typename boost::enable_if<boost::is_enum<Enum> >::type>
   }
 };
 
+template<typename V>
+void FromAnyAction::actId(V& value, const std::string& name, int size)
+{
+  field(*this, value, name, size);
+}
+
+template<class C>
+void FromAnyAction::actId(ptr<C>& value, const std::string& name, int size,
+			  int fkConstraints)
+{
+  actPtr(PtrRef<C>(value, name, size, fkConstraints));
+}
+
 template<typename V>
 void FromAnyAction::act(const FieldRef<V>& field)
 {
