@@ -110,8 +110,7 @@ void PtrRef<C>::visit(A& action, Session *session) const
       if (session)
 	value_ = session->loadLazy<C>(id);
       else
-	throw std::logic_error("Could not load referenced Dbo::ptr, "
-			       "no session?");
+	throw Exception("Could not load referenced Dbo::ptr, no session?");
     }
   }
 }
@@ -147,17 +146,35 @@ void field(A& action, ptr<C>& value, const std::string& name, int size)
   action.actPtr(PtrRef<C>(value, name, size, 0));
 }
 
+template <class A, class C>
+void belongsToImpl(A& action, ptr<C>& value, const std::string& name,
+		   int fkConstraints, int size)
+{
+  if (name.empty() && action.session())
+    action.actPtr(PtrRef<C>(value, action.session()->template tableName<C>(),
+			    size, fkConstraints));
+  else
+    action.actPtr(PtrRef<C>(value, name, size, fkConstraints));
+}
+
 template <class A, class C>
 void belongsTo(A& action, ptr<C>& value, const std::string& name, int size)
 {
-  action.actPtr(PtrRef<C>(value, name, size, 0));
+  belongsToImpl(action, value, name, 0, size);
 }
 
 template <class A, class C>
 void belongsTo(A& action, ptr<C>& value, const std::string& name,
 	       ForeignKeyConstraint constraint, int size)
 {
-  action.actPtr(PtrRef<C>(value, name, size, constraint.value()));
+  belongsToImpl(action, value, name, constraint.value(), size);
+}
+
+template <class A, class C>
+void belongsTo(A& action, ptr<C>& value,
+	       ForeignKeyConstraint constraint, int size)
+{
+  belongsToImpl(action, value, std::string(), constraint.value(), size);
 }
 
 template <class A, class C>
@@ -173,7 +190,9 @@ void hasMany(A& action, collection< ptr<C> >& value,
 	     RelationType type, const std::string& joinName,
 	     const std::string& joinId, ForeignKeyConstraint constraint)
 {
-  assert(type == ManyToMany);
+  if (type != ManyToMany)
+    throw Exception("hasMany() with named joinId only for a ManyToMany relation");
+
   action.actCollection(CollectionRef<C>(value, type, joinName, joinId,
 					constraint.value()));
 }
