@@ -8,7 +8,6 @@
 #define WT_DBO_SESSION_IMPL_H_
 
 #include <iostream>
-#include <stdexcept>
 
 #include <Wt/Dbo/SqlConnection>
 #include <Wt/Dbo/Query>
@@ -41,7 +40,7 @@ template <class C>
 void Session::mapClass(const char *tableName)
 {
   if (schemaInitialized_)
-    throw std::logic_error("Cannot map tables after schema was initialized.");
+    throw Exception("Cannot map tables after schema was initialized.");
 
   if (classRegistry_.find(&typeid(C)) != classRegistry_.end())
     return;
@@ -78,8 +77,8 @@ const char *Session::tableName() const
   if (i != classRegistry_.end())
     return dynamic_cast< Mapping<C> *>(i->second)->tableName;
   else
-    throw std::logic_error(std::string("Class ")
-			   + typeid(C).name() + " was not mapped.");
+    throw Exception(std::string("Class ") + typeid(C).name()
+		    + " was not mapped.");
 }
 
 template <class C>
@@ -95,8 +94,8 @@ Session::Mapping<C> *Session::getMapping() const
       mapping->init(*const_cast<Session *>(this));
     return mapping;
   } else
-    throw std::logic_error(std::string("Class ")
-			   + typeid(C).name() + " was not mapped.");
+    throw Exception(std::string("Class ") + typeid(C).name()
+		    + " was not mapped.");
 }
 
 template <class C>
@@ -174,8 +173,7 @@ ptr<C> Session::add(ptr<C>& obj)
   MetaDbo<C> *dbo = obj.obj();
   if (dbo && !dbo->session()) {
     dbo->setSession(this);
-    dirtyObjects_.insert(dbo);
-    dbo->incRef();
+    needsFlush(dbo);
 
     SessionAddAction act(*dbo, *getMapping<C>());
     act.visit(*dbo->obj());
@@ -192,9 +190,12 @@ ptr<C> Session::add(C *obj)
 }
 
 template <class C>
-ptr<C> Session::load(const typename dbo_traits<C>::IdType& id)
+ptr<C> Session::load(const typename dbo_traits<C>::IdType& id,
+		     bool forceReread)
 {
   ptr<C> result = loadLazy<C>(id);
+  if (forceReread)
+    result.reread();
   *result; // Dereference to do actual load or throw exception
   return result;
 }
@@ -221,7 +222,7 @@ Query< ptr<C>, BindStrategy > Session::find(const std::string& where)
   initSchema();
 
   return Query< ptr<C>, BindStrategy >
-    (*this, Impl::quoteSchemaDot(tableName<C>()), where);
+    (*this, '"' + Impl::quoteSchemaDot(tableName<C>()) + '"', where);
 }
 
 template <class Result>
@@ -250,7 +251,7 @@ template<class C>
 void Session::implSave(MetaDbo<C>& dbo)
 {
   if (!transaction_)
-    throw std::logic_error("Dbo save(): no active transaction");
+    throw Exception("Dbo save(): no active transaction");
 
   if (!dbo.savedInTransaction())
     transaction_->objects_.push_back(new ptr<C>(&dbo));
@@ -267,7 +268,7 @@ template<class C>
 void Session::implDelete(MetaDbo<C>& dbo)
 {
   if (!transaction_)
-    throw std::logic_error("Dbo save(): no active transaction");
+    throw Exception("Dbo save(): no active transaction");
 
   // when saved in transaction, we are already in this list
   if (!dbo.savedInTransaction())
@@ -294,7 +295,6 @@ void Session::implDelete(MetaDbo<C>& dbo)
   statement->execute();
 
   if (versioned) {
-    statement->nextRow();
     int modifiedCount = statement->affectedRowCount();
     if (modifiedCount != 1)
       throw StaleObjectException(boost::lexical_cast<std::string>(dbo.id()),
@@ -305,7 +305,7 @@ void Session::implDelete(MetaDbo<C>& dbo)
 template<class C>
 void Session::implTransactionDone(MetaDbo<C>& dbo, bool success)
 {
-  TransactionDoneAction action(dbo, *getMapping<C>(), success);
+  TransactionDoneAction action(dbo, *this, *getMapping<C>(), success);
   action.visit(*dbo.obj());
 }
 
@@ -313,7 +313,7 @@ template <class C>
 void Session::implLoad(MetaDbo<C>& dbo, SqlStatement *statement, int& column)
 {
   if (!transaction_)
-    throw std::logic_error("Dbo load(): no active transaction");
+    throw Exception("Dbo load(): no active transaction");
 
   LoadDbAction<C> action(dbo, *getMapping<C>(), statement, column);
 
@@ -341,7 +341,7 @@ void Session::Mapping<C>
 ::dropTable(Session& session, std::set<std::string>& tablesDropped)
 {
   if (tablesDropped.count(tableName) == 0) {
-    DropSchema action(session, tableName, tablesDropped);
+    DropSchema action(session, *this, tablesDropped);
     C dummy;
     action.visit(dummy);
   }
