@@ -7,9 +7,8 @@
 #ifndef WT_DBO_COLLECTION_IMPL_H_
 #define WT_DBO_COLLECTION_IMPL_H_
 
-#include <stdexcept>
-
 #include <Wt/Dbo/collection>
+#include <Wt/Dbo/Exception>
 #include <Wt/Dbo/SqlStatement>
 
 namespace Wt {
@@ -67,7 +66,7 @@ collection<C>::iterator::operator* ()
   if (impl_ && !impl_->ended_)
     return impl_->current_;
   else
-    throw std::runtime_error("set< ptr<C> >::operator* : read beyond end.");
+    throw Exception("set< ptr<C> >::operator* : read beyond end.");
 }
 
 template <class C>
@@ -138,7 +137,7 @@ template <class C>
 void collection<C>::iterator::shared_impl::fetchNextRow()
 {
   if (ended_)
-    throw std::runtime_error("set< ptr<C> >::operator++ : beyond end.");
+    throw Exception("set< ptr<C> >::operator++ : beyond end.");
 
   if (!statement_ || !statement_->nextRow()) {
     ended_ = true;
@@ -186,7 +185,7 @@ collection<C>::const_iterator::operator= (const const_iterator& other)
 }
 
 template <class C>
-const C&
+C
 collection<C>::const_iterator::operator* ()
 {
   return impl_.operator*();
@@ -249,6 +248,7 @@ collection<C>::collection()
 {
   data_.relation.sql = 0;
   data_.relation.dbo = 0;
+  data_.relation.setInfo = 0;
   data_.relation.activity = 0;
 }
 
@@ -341,6 +341,12 @@ typename collection<C>::const_iterator collection<C>::end() const
   return const_iterator();
 }
 
+template <class C>
+C collection<C>::front() const
+{
+  return *(const_iterator(*this, executeStatement()));
+}
+
 template <class C>
 typename collection<C>::size_type collection<C>::size() const
 {
@@ -372,14 +378,14 @@ typename collection<C>::size_type collection<C>::size() const
     countStatement->execute();
 
     if (!countStatement->nextRow())
-      throw std::runtime_error("collection<C>::size(): no result?");
+      throw Exception("collection<C>::size(): no result?");
 
     int result;
     if (!countStatement->getResult(0, &result))
-      throw std::runtime_error("collection<C>::size(): null?");
+      throw Exception("collection<C>::size(): null?");
     
     if (countStatement->nextRow())
-      throw std::runtime_error("collection<C>::size(): multiple results?");
+      throw Exception("collection<C>::size(): multiple results?");
 
     if (type_ == QueryCollection) {
       data_.query.size = result;
@@ -395,18 +401,20 @@ template <class C>
 Query<C, DynamicBinding> collection<C>::find() const
 {
   if (type_ != RelationCollection)
-    throw std::runtime_error("collection<C>::find() "
-			     "only for a many-side relation collection.");
+    throw Exception("collection<C>::find() "
+		    "only for a many-side relation collection.");
 
   if (session_ && data_.relation.sql) {
     const std::string *sql = data_.relation.sql;
     std::size_t f = Impl::ifind(*sql, " from ");
     std::size_t w = Impl::ifind(*sql, " where ");
-    std::string tableName = sql->substr(f + 7, w - f - 8);
+    std::string tableName = sql->substr(f + 6, w - f - 6);
 
     Query<C, DynamicBinding> result = Query<C, DynamicBinding>
       (*session_, tableName, "").where(sql->substr(w + 7));
 
+    if (!data_.relation.dbo->isPersisted())
+      data_.relation.dbo->flush();
     data_.relation.dbo->bindId(result.parameters_);
 
     return result;
@@ -417,14 +425,17 @@ Query<C, DynamicBinding> collection<C>::find() const
 template <class C>
 void collection<C>::insert(C c)
 {
-  if (type_ != RelationCollection)
-    throw std::runtime_error("collection<C>::insert() only for a "
-			     "relational collection.");
-
   RelationData& relation = data_.relation;
 
-  if (relation.dbo)
+  if (type_ != RelationCollection || relation.setInfo == 0)
+    throw Exception("collection<C>::insert() only for a relational "
+		    "collection.");
+
+  if (relation.dbo) {
     relation.dbo->setDirty();
+    if (relation.dbo->session())
+      relation.dbo->session()->add(c);
+  }
 
   if (relation.setInfo->type == ManyToMany) {
     if (!relation.activity)
@@ -436,7 +447,8 @@ void collection<C>::insert(C c)
     if (!wasJustErased && !relation.activity->transactionInserted.count(c))
       relation.activity->inserted.insert(c);
   } else {
-    SetReciproceAction setPtr(relation.setInfo->joinName, relation.dbo);
+    SetReciproceAction setPtr(session_, relation.setInfo->joinName,
+			      relation.dbo);
     setPtr.visit(*c.modify());
   }
 }
@@ -444,11 +456,11 @@ void collection<C>::insert(C c)
 template <class C>
 void collection<C>::erase(C c)
 {
-  if (type_ != RelationCollection)
-    throw std::runtime_error("collection<C>::erase() only for a relational "
-			     "relation.");
   RelationData& relation = data_.relation;
 
+  if (type_ != RelationCollection || relation.setInfo == 0)
+    throw Exception("collection<C>::erase() only for a relational relation.");
+
   if (relation.dbo)
     relation.dbo->setDirty();
 
@@ -462,11 +474,33 @@ void collection<C>::erase(C c)
     if (!wasJustInserted && !relation.activity->transactionErased.count(c))
       relation.activity->erased.insert(c);
   } else {
-    SetReciproceAction setPtr(relation.setInfo->joinName, 0);
+    SetReciproceAction setPtr(session_, relation.setInfo->joinName, 0);
     setPtr.visit(*c.modify());
   }
 }
 
+template <class C>
+int collection<C>::count(C c) const
+{
+  if (!session_)
+    throw Exception("collection<C>::count() only for a collection "
+		    "that is bound to a session.");
+
+  session_->flush();
+
+  if (type_ != RelationCollection)
+    throw Exception("collection<C>::count() only for a relational "
+			     "relation.");
+
+  const RelationData& relation = data_.relation;
+  Session::MappingInfo *mapping
+    = session_->getMapping(relation.setInfo->tableName); 
+
+  Query<C, DynamicBinding> q = find().where(mapping->idCondition);
+  c.obj()->bindId(q.parameters_);
+  return q.resultList().size();
+}
+
 template <class C>
 void collection<C>::resetActivity()
 {
