@@ -62,17 +62,18 @@ public:
   HeaderMap headerMap;
   std::vector<HeaderMap::iterator> headerOrder;
   ::int64_t contentLength;
+  int webSocketVersion;
 
   std::string request_path;
   std::string request_query;
   std::string request_extra_path;
-  bool webSocketRequest;
 
   void reset();
 
   bool closeConnection() const;
   bool acceptGzipEncoding() const;
   void enableWebSocket();
+  std::string getHeader(const std::string& name) const;
 
   void transmitHeaders(std::ostream& out) const;
 };
