@@ -57,6 +57,10 @@ private:
   virtual void startAsyncReadBody(Buffer& buffer, int timeout);
   virtual void startAsyncWriteResponse
       (const std::vector<asio::const_buffer>& buffers, int timeout);
+  void handleReadRequestSsl(const asio_error_code& e,
+                            std::size_t bytes_transferred);
+  void handleReadBodySsl(const asio_error_code& e,
+                         std::size_t bytes_transferred);
 
 private:
   void handleHandshake(const asio_error_code& error);
