@@ -8,16 +8,16 @@
 
 WT_DECLARE_WT_MEMBER
 (1, JavaScriptConstructor, "StdLayout",
- function(WT, id, config) {
+ function(WT, id, fitHeight, config) {
    var self = this;
    var initialized = false;
 
+   this.WT = WT;
+
    this.getId = function() {
      return id;
    };
 
-   this.WT = WT;
-
    this.marginH = function(el) {
        var p = el.parentNode;
        var result = 0;
@@ -40,7 +40,9 @@ WT_DECLARE_WT_MEMBER
        var result = WT.px(el, 'marginTop');
        result += WT.px(el, 'marginBottom');
 
-       if (!WT.boxSizing(el)) {
+       /* Second condition: IE9 applys boxsizing to 'BUTTON' objects ? */
+       if (!WT.boxSizing(el)
+	   && !(WT.isIE && !WT.isIElt9 && WT.hasTag(el, 'BUTTON'))) {
 	   result += WT.px(el, 'borderTopWidth');
 	   result += WT.px(el, 'borderBottomWidth');
 	   result += WT.px(el, 'paddingTop');
@@ -93,8 +95,6 @@ WT_DECLARE_WT_MEMBER
        return;
 
      var ch = td.childNodes[0]; // 'ch' is cell contents
-     if (height <= 0)
-       height = 0;
 
      if (ch.className == 'Wt-hcenter') {
        ch.style.height = height+'px';
@@ -106,11 +106,10 @@ WT_DECLARE_WT_MEMBER
        ch = itd.firstChild;
      }
 
-     if (td.childNodes.length == 1)
-       height -= this.marginV(ch);
+     height -= this.marginV(ch);
 
-       if (height <= 0)
-	 height = 0;
+     if (height <= 0)
+       height = 0;
 
      if (WT.hasTag(ch, 'TABLE'))
        return;
@@ -121,7 +120,7 @@ WT_DECLARE_WT_MEMBER
        p = ch.parentNode;
        w = p.offsetWidth - self.marginH(ch);
 
-       if (p.className == 'Wt-chwrap') {
+       if (p.className == 'Wt-chwrap' && w > 0) {
 	 if (!WT.isIE) {
 	   ch.style.position = 'relative';
 	   ch = ch.firstChild;
@@ -189,11 +188,16 @@ WT_DECLARE_WT_MEMBER
      if (t.style.height !== '')
        t.style.height = '';
 
-     var pHeight = WT.pxself(p, 'height'), usingClientHeight = false;
+     var pHeight, usingClientHeight = false;
+     if (fitHeight) {
+       pHeight = WT.pxself(p, 'height');
 
-     if (pHeight === 0) {
-       usingClientHeight = true;
-       pHeight = p.clientHeight;
+       if (pHeight === 0) {
+	 usingClientHeight = true;
+	 pHeight = p.clientHeight;
+       }
+     } else {
+       pHeight = t.clientHeight;
      }
 
      var pWidth = p.clientWidth;
@@ -217,59 +221,61 @@ WT_DECLARE_WT_MEMBER
       * remove padding of the parent, and margin of myself.
       */
      var r = pHeight;
-     if (usingClientHeight) {
-       r -= WT.px(p, 'paddingTop');
-       r -= WT.px(p, 'paddingBottom');
-     } else if (WT.boxSizing(p)) {
-       r -= WT.px(p, 'borderTopWidth');
-       r -= WT.px(p, 'borderBottomWidth');
-       r -= WT.px(p, 'paddingTop');
-       r -= WT.px(p, 'paddingBottom');
-     }
 
-     r -= WT.px(widget, 'marginTop');
-     r -= WT.px(widget, 'marginBottom');
+     var i, il = t.rows.length,
+         ri, row, rowi; // iterator variables
 
-     var i, il;
+     if (fitHeight) {
+       if (usingClientHeight) {
+	 r -= WT.px(p, 'paddingTop');
+	 r -= WT.px(p, 'paddingBottom');
+       } else if (WT.boxSizing(p)) {
+	 r -= WT.px(p, 'borderTopWidth');
+	 r -= WT.px(p, 'borderBottomWidth');
+	 r -= WT.px(p, 'paddingTop');
+	 r -= WT.px(p, 'paddingBottom');
+       }
 
-     /*
-      * Sometimes, there may be other elements; e.g. in FIELDSET.
-      * Remove the height of these too
-      */
-     if (p.children && p.children.length !== 1) {
-       for (i = 0, il = p.children.length; i < il; ++i) {
-	 var w = p.children[i];
+       r -= WT.px(widget, 'marginTop');
+       r -= WT.px(widget, 'marginBottom');
+
+       /*
+	* Sometimes, there may be other elements; e.g. in FIELDSET.
+	* Remove the height of these too
+	*/
+       if (p.children && p.children.length !== 1) {
+	 for (i = 0, il = p.children.length; i < il; ++i) {
+	   var w = p.children[i];
 	   if (w !== widget)
 	     r -= $(w).outerHeight();
-       }
-     }
-
-     /*
-      * Reduce 'r' with the total height of rows with stretch=0.
-      */
-     var ts = 0,              // Sum of stretch factors
-         tmh = 0,             // Min heights
-         ri, j, jl, row, tds; // Iterator variables
-
-     il = t.rows.length;
-
-     for (i = 0, ri = 0; i < il; i++) {
-       row = t.rows[i];
-
-       if (row.className === 'Wt-hrh') {  // Skip resize rows
-	 r -= row.offsetHeight;          // Reduce r
-	 continue;
+	 }
        }
 
-       tmh += config.minheight[ri];
-       if (config.stretch[ri] <= 0)
-	 r -= row.offsetHeight;          // Reduce r
-       else
-	 ts += config.stretch[ri];
-       ++ri;
-     }
+       /*
+	* Reduce 'r' with the total height of rows with stretch=0.
+	*/
+       var ts = 0,              // Sum of stretch factors
+           tmh = 0,             // Min heights
+	   j, jl, tds;          // Iterator variables
 
-     r = r > tmh ? r : tmh;
+       for (i = 0, ri = 0; i < il; i++) {
+	 row = t.rows[i];
+
+	 if (row.className) {     // Skip special rows
+	   r -= row.offsetHeight; // Reduce r
+	   continue;
+	 }
+
+	 tmh += config.minheight[ri];
+	 if (config.stretch[ri] <= 0)
+	   r -= row.offsetHeight;          // Reduce r
+	 else
+	   ts += config.stretch[ri];
+	 ++ri;
+       }
+
+       r = r > tmh ? r : tmh;
+     }
 
      var rowspan_tds = [];
 
@@ -278,39 +284,37 @@ WT_DECLARE_WT_MEMBER
       *  for every row (which has a stretch) and for every cell. Apply the
       *  same height to each cell's contents as well
       */
-     if (ts !== 0 && r > 0) {
-       var left = r, // remaining space to be divided
-           stretch, mh, h;
+     var left = r, // remaining space to be divided
+         stretch, mh, h;
 
-       for (i = 0, ri = 0; i < il; i++) {
-	 row = t.rows[i];
+     for (i = 0, ri = 0; i < il; i++) {
+       row = t.rows[i];
 
-	 if (row.className === 'Wt-hrh') // Skip resize rows
-	   continue;
+       if (row.className)
+	 continue;
 
-	 stretch = config.stretch[ri];
-	 if (stretch !== 0) {
-	   /*
-	    * The target height 'h', cannot be more than what is still
-	    * left to distribute, and cannot be less than the minimum height
-	    */
+       stretch = config.stretch[ri];
+       if (stretch == -1 || (fitHeight && stretch > 0)) {
+	 /*
+	  * The target height 'h', cannot be more than what is still
+	  * left to distribute, and cannot be less than the minimum height
+	  */
 
-	   if (stretch !== -1) {
-	     mh = config.minheight[ri];
+	 if (stretch !== -1) {
+	   mh = config.minheight[ri];
 
-	     h = r * stretch / ts;
-	     h = left > h ? h : left;
-	     h = Math.round(mh > h ? mh : h);
-	     left -= h;
-	   } else {
-	     h = row.offsetHeight;
-	   }
-
-	   WT.addAll(rowspan_tds, this.adjustRow(row, h));
+	   h = r * stretch / ts;
+	   h = left > h ? h : left;
+	   h = Math.round(mh > h ? mh : h);
+	   left -= h;
+	 } else {
+	   h = row.offsetHeight;
 	 }
 
-	 ++ri;
+	 WT.addAll(rowspan_tds, this.adjustRow(row, h));
        }
+
+       ++ri;
      }
 
      for (i = 0, il = rowspan_tds.length; i < il; ++i) {
@@ -421,16 +425,15 @@ WT_DECLARE_WT_MEMBER
      for (i=0, ri=0, il=t.rows.length; i<il; i++) {
        row = t.rows[i];
 
-       if (row.className != 'Wt-hrh') {
+       if (!row.className) {
 	 var td, j, ci, jl;
 	 for (j=0, ci=0, jl=row.childNodes.length; j<jl; ++j) {
 	   td = row.childNodes[j];
 
 	   if (td.className != 'Wt-vrh') {
-	     if (td.colSpan == 1 && ci == columni && td.childNodes.length==1) {
+	     if (td.colSpan == 1 && ci == columni && td.childNodes.length > 0) {
 	       var ch = td.firstChild;
-	       var w = width - self.marginH(ch);
-
+	       var w = Math.max(1, width - self.marginH(ch));
 	       ch.style.width = w + 'px';
 	       break;
 	     }
@@ -572,13 +575,15 @@ WT_DECLARE_WT_MEMBER
    for (i=0, ri=0, il=t.rows.length; i<il; i++) {
      row = t.rows[i];
 
-     if (row.className == 'Wt-hrh') {
-       var td = row.firstChild;
-       td.ri = ri - 1;
-       td.onmousedown = td.ontouchstart = function(event) {
-	 var e = event||window.event;
-	 startRowResize(this, this.ri, e);
-       };
+     if (row.className) {
+       if (row.className === 'Wt-hrh') {
+	 var td = row.firstChild;
+	 td.ri = ri - 1;
+	 td.onmousedown = td.ontouchstart = function(event) {
+	   var e = event||window.event;
+	   startRowResize(this, this.ri, e);
+	 };
+       }
      } else {
        var td, j, ci, jl;
        for (j=0, ci=0, jl=row.childNodes.length; j<jl; ++j) {
