@@ -25,8 +25,10 @@ WT_DECLARE_WT_MEMBER
      if (wxy.x > 0 && wxy.x < wsize.x && wxy.y > 0 && wxy.y < wsize.y) {
        centerX = centerY = false;
 
-       el.style.left = (WT.pxself(el, 'left') + nowxy.x - dsx) + 'px';
-       el.style.top = (WT.pxself(el, 'top') + nowxy.y - dsy) + 'px';
+       el.style.left = (WT.px(el, 'left') + nowxy.x - dsx) + 'px';
+       el.style.top = (WT.px(el, 'top') + nowxy.y - dsy) + 'px';
+       el.style.right = '';
+       el.style.bottom = '';
        dsx = nowxy.x;
        dsy = nowxy.y;
      }
@@ -60,13 +62,17 @@ WT_DECLARE_WT_MEMBER
        var ws = WT.windowSize();
        var w = el.offsetWidth, h = el.offsetHeight;
 
-       if (centerX)
+       if (centerX) {
 	 el.style.left = Math.round((ws.x - w)/2
 	     + (WT.isIE6 ? document.documentElement.scrollLeft : 0)) + 'px';
+	 el.style.marginLeft = '0px';
+       }
 
-       if (centerY)
+       if (centerY) {
 	 el.style.top = Math.round((ws.y - h)/2
 	     + (WT.isIE6 ? document.documentElement.scrollTop : 0)) + 'px';
+	 el.style.marginTop = '0px';
+       }
 
        if (el.style.height != '')
 	 wtResize(el, -1, h);
@@ -77,12 +83,13 @@ WT_DECLARE_WT_MEMBER
 
    function wtResize(self, w, h) {
      h -= 2; w -= 2; // 2 = dialog border
-     self.style.height = h + 'px';
+     self.style.height = Math.max(0, h) + 'px';
      if (w > 0)
-       self.style.width = w + 'px';
-     var c = self.lastChild;
-     var t = c.previousSibling;
-     h -= t.offsetHeight + 8; // 8 = body padding
+       self.style.width = Math.max(0, w) + 'px';
+     var c = $(self).children('.body').get(0);
+     var t = $(self).children('.titlebar').get(0);
+     if (t)
+       h -= t.offsetHeight + 8; // 8 = body padding
      if (h > 0) {
        c.style.height = h + 'px';
        if (APP.layouts)
@@ -90,6 +97,11 @@ WT_DECLARE_WT_MEMBER
      }
    };
 
+   this.onresize = function(w, h) {
+     centerX = centerY = false;
+     wtResize(el, w, h);
+   };
+
    el.wtResize = wtResize;
    el.wtPosition = this.centerDialog;
  });
