@@ -51,24 +51,43 @@ this.condCall = function(o, f, a) {
 // buttons currently down
 this.buttons = 0;
 
+// button last released (for reporting in IE's click event)
+var lastButtonUp = 0;
+
 // returns the button associated with the event (0 if none)
 this.button = function(e)
 {
-  if (e.which) {
+  try {
+    var t = e.type;
+
+    if (WT.isIE && (t == "click" || t == "dblclick"))
+      return lastButtonUp; // IE does not provide button information then
+
+    if (t != "mouseup" && t != "mousedown" && t != "click" && t != "dblclick")
+      return 0;
+  } catch (e) {
+    return 0;
+  }
+
+  if (!WT.isGecko && typeof e.which !== 'undefined') {
     if (e.which == 3)
       return 4;
     else if (e.which == 2)
       return 2;
-    else
+    else if (e.which == 1)
       return 1;
-  } else if (WT.isIE && typeof e.button != 'undefined') {
+    else
+      return 0;
+  } else if (WT.isIE && typeof e.button !== 'undefined') {
     if (e.button == 2)
       return 4;
     else if (e.button == 4)
       return 2;
-    else
+    else if (e.button == 1)
       return 1;
-  } else if (typeof e.button != 'undefined') {
+    else
+      return 0;
+  } else if (typeof e.button !== 'undefined') {
     if (e.button == 2)
       return 4;
     else if (e.button == 1)
@@ -86,7 +105,8 @@ this.mouseDown = function(e) {
 };
 
 this.mouseUp = function(e) {
-  WT.buttons &= ~WT.button(e);
+  lastButtonUp = WT.button(e);
+  WT.buttons &= ~lastButtonUp;
 };
 
 /**
@@ -123,13 +143,11 @@ this.isIE = ie !== undefined;
 this.isIE6 = ie === 6;
 this.isIElt9 = ie < 9;
 this.isIEMobile = agent.indexOf("msie 4")!=-1 || agent.indexOf("msie 5")!=-1;
-this.isGecko = agent.indexOf("gecko") != -1;
 this.isOpera = typeof window.opera !== 'undefined';
 this.isAndroid = (agent.indexOf("safari") != -1)
 		  && (agent.indexOf("android") != -1);
-this.isMobileWebKit = (agent.indexOf("applewebkit") != -1)
-		       && (agent.indexOf("mobile") != -1);
 this.isWebKit = (agent.indexOf("applewebkit") != -1);
+this.isGecko = agent.indexOf("gecko") != -1 && !this.isWebKit;
 
 this.updateDelay = this.isIE ? 10 : 51;
 
@@ -231,15 +249,17 @@ this.initAjaxComm = function(url, handler) {
 	  else
 	    handler(1, null, userData);
 
-	  request.onreadystatechange = new Function;
-	  try {
-	    request.onload = request.onreadystatechange;
-	  } catch (e) {
-	    /*
-	     * See comment below.
-	     */
+	  if (request) {
+	    request.onreadystatechange = new Function;
+	    try {
+	      request.onload = request.onreadystatechange;
+	    } catch (e) {
+	      /*
+	       * See comment below.
+	       */
+	    }
+	    request = null;
 	  }
-	  request = null;
 
 	  handled = true;
 	}
@@ -299,7 +319,7 @@ this.initAjaxComm = function(url, handler) {
 
       this.setUrl = function(url) {
 	sessionUrl = url;
-      }
+      };
     })();
   } else {
     return new (function() {
@@ -353,7 +373,8 @@ this.initAjaxComm = function(url, handler) {
 
       this.setUrl = function(url) {
 	sessionUrl = url;
-      }
+      };
+
     })();
   }
 };
@@ -456,6 +477,31 @@ this.unstub = function(from, to, methodDisplay) {
     to.style.width = from.style.width;
 };
 
+this.changeTag = function(e, type)
+{
+  var n = document.createElement(type);
+
+   /* For some reason fails on 'a' */
+  if (type == 'img' && n.mergeAttributes) {
+    n.mergeAttributes(e, false);
+    n.src = e.src;
+  } else {
+    if (e.attributes && e.attributes.length > 0) {
+      var i, il;
+      for (i = 0, il = e.attributes.length; i < il; i++) {
+	var nn = e.attributes[i].nodeName;
+	if (nn != 'type' && nn != 'name')
+	  n.setAttribute(nn, e.getAttribute(nn));
+      }
+    }
+  }
+
+  while (e.firstChild)
+    n.appendChild(e.removeChild(e.firstChild));
+
+  e.parentNode.replaceChild(n, e);
+};
+
 this.unwrap = function(e) {
   e = WT.getElement(e);
   if (!e.parentNode.className.indexOf('Wt-wrap')) {
@@ -463,32 +509,20 @@ this.unwrap = function(e) {
     e = e.parentNode;
     if (e.className.length >= 8)
       wrapped.className = e.className.substring(8);
-    if (WT.isIE)
-      wrapped.style.setAttribute('cssText', e.getAttribute('style'));
-    else
-      wrapped.setAttribute('style', e.getAttribute('style'));
+    var style = e.getAttribute('style');
+    if (style) {
+      if (WT.isIE)
+	wrapped.style.setAttribute('cssText', style);
+      else
+	wrapped.setAttribute('style', style);
+    }
     e.parentNode.replaceChild(wrapped, e);
   } else {
     if (e.getAttribute('type') == 'submit') {
       e.setAttribute('type', 'button');
       e.removeAttribute('name');
     } if (WT.hasTag(e, 'INPUT') && e.getAttribute('type') == 'image') {
-      // change <input> to <image>
-      var img = document.createElement('img');
-      if (img.mergeAttributes) {
-	img.mergeAttributes(e, false);
-	img.src = e.src;
-      } else {
-	if (e.attributes && e.attributes.length > 0) {
-	  var i, il;
-	  for (i = 0, il = e.attributes.length; i < il; i++) {
-	    var n = e.attributes[i].nodeName;
-	    if (n != 'type' && n != 'name')
-	      img.setAttribute(n, e.getAttribute(n));
-	  }
-	}
-      }
-      e.parentNode.replaceChild(img, e);
+      WT.changeTag(e, 'img');
     }
   }
 };
@@ -522,7 +556,7 @@ this.ajaxInternalPaths = function(basePath) {
 	internalPath = href.substr(absBase.length - 1);
       } else
 	internalPath = href.substr(basePath.length);
-	
+
       if (internalPath.substr(0, 3) == "?_=")
 	  internalPath = internalPath.substr(3);
       this.setAttribute('href', href); // computes this.href
@@ -588,7 +622,13 @@ this.getElement = function(id) {
 };
 
 this.validate = function(edit) {
-  var v = edit.wtValidate.validate(edit.value);
+  var v;
+  if (edit.options)
+    v = edit.options.item(edit.selectedIndex).text;
+  else
+    v = edit.value;
+
+  v = edit.wtValidate.validate(v);
   if (v.valid) {
     edit.removeAttribute('title');
     $(edit).removeClass('Wt-invalid');
@@ -620,7 +660,7 @@ this.widgetPageCoordinates = function(obj) {
     objX += obj.offsetLeft;
     objY += obj.offsetTop;
 
-    var f = css(obj, 'position');
+    var f = WT.css(obj, 'position');
     if (f == 'fixed') {
       objX += document.body.scrollLeft
 	+ document.documentElement.scrollLeft;
@@ -637,7 +677,17 @@ this.widgetPageCoordinates = function(obj) {
       do {
 	obj = obj.parentNode;
 	if (WT.hasTag(obj, "DIV")) {
-	  if (!rtl) /* Somebody explain me why, but that seems to work +/- */
+	  if (rtl && !WT.isGecko) {
+	    if (obj.scrollWidth > obj.parentNode.scrollWidth) {
+	      /*
+	       * This seems to be a bug in every browser out there,
+	       * except for Gecko ?
+	       */
+	      var sl = obj.scrollLeft
+		+ obj.parentNode.scrollWidth - obj.scrollWidth;
+	      objX -= sl;
+	    }
+	  } else
 	    objX -= obj.scrollLeft;
 	  objY -= obj.scrollTop;
 	}
@@ -829,7 +879,7 @@ this.stopRepeat = function() {
 
 var cacheC = null, cacheS = null;
 
-function css(c, s) {
+this.css = function(c, s) {
   if (c.style[s])
     return c.style[s];
   else {
@@ -846,7 +896,7 @@ function css(c, s) {
 
     return cacheS ? cacheS[s] : null;
   }
-}
+};
 
 function parseCss(value, regex, defaultvalue) {
   if (value == 'auto' || value == null)
@@ -856,9 +906,9 @@ function parseCss(value, regex, defaultvalue) {
   return v ? parseFloat(v) : defaultvalue;
 }
 
-function parsePx(v) {
+this.parsePx = function(v) {
   return parseCss(v, /^\s*(-?\d+(?:\.\d+)?)\s*px\s*$/i, 0);
-}
+};
 
 function parsePct(v, defaultValue) {
   return parseCss(v, /^\s*(-?\d+(?:\.\d+)?)\s*\%\s*$/i, defaultValue);
@@ -866,12 +916,12 @@ function parsePct(v, defaultValue) {
 
 // Get an element metric in pixels
 this.px = function(c, s) {
-  return parsePx(css(c, s));
+  return WT.parsePx(WT.css(c, s));
 };
 
 // Get a widget style in pixels, when set directly
 this.pxself = function(c, s) {
-  return parsePx(c.style[s]);
+  return WT.parsePx(c.style[s]);
 };
 
 this.pctself = function(c, s) {
@@ -908,6 +958,24 @@ this.isHidden = function(w) {
   }
 };
 
+this.innerWidth = function(el) {
+  var result = el.offsetWidth;
+  if (!WT.boxSizing(el)) {
+    result -= WT.px(el, 'paddingLeft') + WT.px(el, 'paddingRight')
+	  + WT.px(el, 'borderLeftWidth') + WT.px(el, 'borderRightWidth');
+  }
+  return result;
+};
+
+this.innerHeight = function(el) {
+  var result = el.offsetHeight;
+  if (!WT.boxSizing(el)) {
+    result -= WT.px(el, 'paddingTop') + WT.px(el, 'paddingBottom')
+	  + WT.px(el, 'borderTopWidth') + WT.px(el, 'borderBottomWidth');
+  }
+  return result;
+};
+
 this.IEwidth = function(c, min, max) {
   if (c.parentNode) {
     var r = c.parentNode.clientWidth
@@ -1048,7 +1116,7 @@ this.capture = function(obj) {
   captureElement = obj;
 
   var db = document.body;
-  if (db.setCapture)
+  if (!document.body.addEventListener)
     if (obj != null)
       db.setCapture();
     else
@@ -1086,9 +1154,33 @@ this.getElementsByClassName = function(className, parentElement) {
 };
 
 /* Firefox, IE9 etc... */
+var inlineStyleSheet = null;
+
+function getInlineStyleSheet() {
+  if (!inlineStyleSheet) {
+    var i, il, ds = document.styleSheets;
+    for (i = 0, il = ds.length; i < il; ++i) {
+      var s = ds[i];
+      if (WT.hasTag(ds[i].ownerNode, 'STYLE')) {
+	inlineStyleSheet = s;
+	break;
+      }
+    }
+
+    if (!inlineStyleSheet) {
+      var s = document.createElement('style');
+      document.getElementsByTagName('head')[0].appendChild(s);
+
+      inlineStyleSheet = s.sheet;
+    }
+  }
+
+  return inlineStyleSheet;
+}
+
 this.addCss = function(selector, style) {
-   // on IE we have the VML too
-  var s = document.styleSheets[WT.isIE ? 1 : 0];
+  var s = getInlineStyleSheet();
+
   // strange error with IE9 when in iframe
   var pos = s.cssRules ? s.cssRules.length : 0;
   s.insertRule(selector + ' { ' + style + ' }', pos);
@@ -1173,14 +1265,18 @@ this.addStyleSheet = function(uri, media) {
     setTimeout(function() { document.createStyleSheet(uri); }, 15);
   } else {
     var s = document.createElement('link');
-    s.setAttribute('type', 'text/css');
     s.setAttribute('href', uri);
     s.setAttribute('type','text/css');
     s.setAttribute('rel','stylesheet');
     if (media != '' && media != 'all')
       s.setAttribute('media', media);
-    var h = document.getElementsByTagName('head')[0];
-    h.appendChild(s);
+    var ll = document.getElementsByTagName('link');
+    if (ll.length > 0) {
+      var l = ll[ll.length - 1];
+      l.parentNode.insertBefore(s, l.nextSibling);
+    } else {
+      document.body.appendChild(s);
+    }
   }
 };
 
@@ -1311,13 +1407,29 @@ this.hasFocus = function(el) {
   return el == document.activeElement;
 };
 
-var html5History = !WT.isMobileWebKit
-    && !!(window.history && window.history.pushState);
+var html5History = !!(window.history && window.history.pushState);
+
+/*
+ * A less aggressive URL encoding than encodeURIComponent which does
+ * for example not encode '/'
+ */
+function gentleURIEncode(s) {
+  return s.replace(/%/g, '%25')
+    .replace(/\+/g, '%2b')
+    .replace(/ /g, '%20')
+    .replace(/#/g, '%23')
+    .replace(/&/g, '%26');
+}
 
 if (html5History) {
   this.history = (function()
 {
-  var currentState = null, baseUrl = null, cb = null;
+  var currentState = null, baseUrl = null, cb = null, stateMap = { },
+      w = window;
+
+  function saveState(state) {
+    stateMap[w.location.pathname + w.location.search] = state;
+  }
 
   return {
     _initialize: function() { },
@@ -1327,24 +1439,18 @@ if (html5History) {
     register: function (initialState, onStateChange) {
       currentState = initialState;
       cb = onStateChange;
-      var expectNullState = WT.isWebKit;
+      saveState(initialState);
 
       function onPopState(event) {
 	var newState = event.state;
 
-	/*
-	 * A null state event is given onload, but we may already have
-	 * pushed another state... this is simply silly.
-	 *
-	 * see http://html5.org/tools/web-apps-tracker?from=5345&to=5346
-	 */
+	if (newState === null)
+	  newState = stateMap[w.location.pathname + w.location.search];
 
-	if (!newState)
-	  if (expectNullState) {
-	    expectNullState = false;
-	    return;
-	  } else
-	    newState = initialState;
+	if (newState === null) {
+	  saveState(currentState);
+	  return;
+	}
 
 	if (newState != currentState) {
 	  currentState = newState;
@@ -1352,37 +1458,21 @@ if (html5History) {
 	}
       }
 
-      function onHashChange() {
-	var p = window.location.hash;
-	var newState = null;
-	if (p == '')
-	  newState = p;
-	else if (p.substr(0, 2) == '#/')
-	  newState = p.substr(2);
-	if (newState !== currentState) {
-	  currentState = newState;
-	  onStateChange(currentState);
-	}
-      }
-
       window.addEventListener("popstate", onPopState, false);
-      window.addEventListener("hashchange", onHashChange, false);
     },
 
     initialize: function (stateField, histFrame, deployUrl) {
       WT.resolveRelativeAnchors();
 
-      if (deployUrl && deployUrl[deployUrl.length - 1] == '/') {
+      baseUrl = deployUrl;
+      if (baseUrl.length >= 1 && baseUrl[baseUrl.length - 1] == '/') {
 _$_$if_UGLY_INTERNAL_PATHS_$_();
-	baseUrl = deployUrl + "?_=";
+	baseUrl += "?_=";
 _$_$endif_$_();
 _$_$ifnot_UGLY_INTERNAL_PATHS_$_();
-	baseUrl = deployUrl.substr(0, deployUrl.length - 1);
+	baseUrl = baseUrl.substr(0, baseUrl.length - 1);
 _$_$endif_$_();
-      } else if (deployUrl)
-	baseUrl = deployUrl;
-      else
-	baseUrl = window.location.pathname;
+      }
     },
 
     navigate: function (state, generateEvent) {
@@ -1390,16 +1480,17 @@ _$_$endif_$_();
 
       currentState = state;
 
-      var url = baseUrl + state;
+      var url = baseUrl + gentleURIEncode(state);
 
-      if (baseUrl.length < 3 || baseUrl.substr(baseUrl.length - 3) != "?_=")
+      if (baseUrl.length < 3 || baseUrl.substr(baseUrl.length - 3) != "?_=") {
 	url += window.location.search;
-      else {
+      } else {
 	function stripHashParameter(q) {
 	  if (q.length > 1)
 	    q = q.substr(1);
 
-	  var qp = q.split("&"), i, il, q="";
+	  var qp = q.split("&"), i, il;
+	  q = "";
 
 	  for (i=0, il = qp.length; i<il; ++i)
 	    if (qp[i].split("=")[0] != '_')
@@ -1409,8 +1500,8 @@ _$_$endif_$_();
 	}
 
 	var q = stripHashParameter(window.location.search);
-	if (q.length > 0)
-	  url += q.substr(1);
+	if (q.length > 1)
+	  url += '&' + q.substr(1);
       }
 
       try {
@@ -1423,6 +1514,8 @@ _$_$endif_$_();
 	console.log(error.toString());
       }
 
+      WT.scrollIntoView(state);
+
       if (generateEvent)
 	cb(state);
     },
@@ -1637,6 +1730,8 @@ _$_$endif_$_();
     }
   },
   navigate: function (state, generateEvent) {
+    state = gentleURIEncode(state);
+
     if (!_initialized) {
       return;
     }
@@ -1644,7 +1739,8 @@ _$_$endif_$_();
     if (_UAie) {
       _updateIFrame(fqstate);
     } else {
-      location.hash = fqstate;
+      if (fqstate.length > 0)
+	location.hash = fqstate;
       if (_UAwebkit) {
 	_fqstates[history.length] = fqstate;
 	_storeStates();
@@ -1657,7 +1753,7 @@ _$_$endif_$_();
     if (!_initialized) {
       return "";
     }
-    return _currentState;
+    return unescape(_currentState);
   }
   };
 })();
@@ -1681,6 +1777,8 @@ var WT = _$_WT_CLASS_$_;
 var downX = 0;
 var downY = 0;
 
+var deployUrl = _$_DEPLOY_PATH_$_;
+
 function saveDownPos(e) {
   var coords = WT.pageCoordinates(e);
   downX = coords.x;
@@ -1690,13 +1788,16 @@ function saveDownPos(e) {
 var currentHash = null;
 
 function onHashChange() {
-  var newLocation = WT.history.getCurrentState();
-  if (currentHash == newLocation) {
+  var newLocation = _$_WT_CLASS_$_.history.getCurrentState();
+
+  if (newLocation.length > 0 && newLocation.substr(0, 1) != '/')
     return;
-  } else {
-    currentHash = newLocation;
-    setTimeout(function() { update(null, 'hash', null, true); }, 1);
-  }
+
+  if (currentHash == newLocation)
+    return;
+
+  currentHash = newLocation;
+  setTimeout(function() { update(null, 'hash', null, true); }, 1);
 };
 
 function setHash(newLocation) {
@@ -1705,7 +1806,7 @@ function setHash(newLocation) {
 
   currentHash = newLocation;
 
-  WT.history.navigate(escape(newLocation), false);
+  WT.history.navigate(newLocation, false);
 };
 
 var dragState = {
@@ -1754,6 +1855,7 @@ function dragStart(obj, e) {
   ds.object.parentNode.removeChild(ds.object);
   ds.object.style["position"] = 'absolute';
   ds.object.className = '';
+  ds.object.style["z-index"] = '1000';
   document.body.appendChild(ds.object);
 
   WT.capture(null);
@@ -1908,7 +2010,9 @@ function encodeEvent(event, i) {
     if (el == null)
       continue;
 
-    if (el.type == 'select-multiple') {
+    if (el.wtEncodeValue)
+      v = el.wtEncodeValue(el);
+    else if (el.type == 'select-multiple') {
       for (j = 0, jl = el.options.length; j < jl; j++)
 	if (el.options[j].selected) {
 	  result += se + formObjects[x] + '='
@@ -1919,24 +2023,6 @@ function encodeEvent(event, i) {
 	v = 'i';
       else if (el.checked)
 	v = el.value;
-    } else if (WT.hasTag(el, "VIDEO") || WT.hasTag(el, "AUDIO")) {
-      v = '' + el.volume + ';'
-	+ el.currentTime + ';'
-	+ el.duration + ';'
-	+ (el.paused ? '1' : '0') + ';'
-	+ (el.ended ? ' 1' : '0') + ';'
-	+ el.readyState;
-    } else if (WT.hasTag(el, "DIV") && el.childNodes.length == 1) {
-      // When in Layout, media elements sits in a surrounding DIV
-      var m = el.childNodes[0];
-      if (WT.hasTag(m, "VIDEO") || WT.hasTag(m, "AUDIO")) {
-        v = '' + m.volume + ';'
-	  + m.currentTime + ';'
-	  + m.duration + ';'
-	  + (m.paused ? '1' : '0') + ';'
-	  + (m.ended ? '1' : '0') + ';'
-	  + m.readyState;
-      }
     } else if (el.type != 'file') {
       if ($(el).hasClass('Wt-edit-emptyText'))
 	v = '';
@@ -1965,7 +2051,7 @@ function encodeEvent(event, i) {
   } catch (e) { }
 
   if (currentHash != null)
-    result += se + '_=' + encodeURIComponent(unescape(currentHash));
+    result += se + '_=' + encodeURIComponent(currentHash);
 
   if (!e) {
     event.data = result;
@@ -2072,7 +2158,7 @@ function encodePendingEvents() {
   return { feedback: feedback, result: result };
 }
 
-var sessionUrl = _$_RELATIVE_URL_$_,
+var sessionUrl,
   quited = false,
   norestart = false,
   loaded = false,
@@ -2117,8 +2203,7 @@ function load(fullapp) {
     if (!window._$_APP_CLASS_$_LoadWidgetTree)
       return; // That's too soon baby.
 
-    WT.history.initialize("Wt-history-field", "Wt-history-iframe",
-			  _$_DEPLOY_PATH_$_);
+    WT.history.initialize("Wt-history-field", "Wt-history-iframe", deployUrl);
   }
 
   if (!("activeElement" in document)) {
@@ -2136,6 +2221,8 @@ function load(fullapp) {
     document.addEventListener("blur", trackActiveElementLost, true);
   }
 
+  $(document).mousedown(WT.mouseDown).mouseup(WT.mouseUp);
+
   WT.history._initialize();
   initDragDrop();
   loaded = true;
@@ -2144,15 +2231,16 @@ function load(fullapp) {
     window._$_APP_CLASS_$_LoadWidgetTree();
 
   if (!quited) {
-    doKeepAlive();
-    keepAliveTimer = setInterval(doKeepAlive, _$_KEEP_ALIVE_$_000);
+    if (!keepAliveTimer) {
+      keepAliveTimer = setInterval(doKeepAlive, _$_KEEP_ALIVE_$_000);
+    }
   }
 }
 
 var currentHideLoadingIndicator = null;
 
-function cancelFeedback(t) {
-  clearTimeout(t);
+function cancelFeedback(timer) {
+  clearTimeout(timer);
   document.body.style.cursor = 'auto';
 
   if (currentHideLoadingIndicator != null) {
@@ -2214,6 +2302,7 @@ function handleResponse(status, msg, timer) {
     return;
 
   if (status == 0) {
+    WT.resolveRelativeAnchors();
 _$_$if_CATCH_ERROR_$_();
     try {
 _$_$endif_$_();
@@ -2263,13 +2352,20 @@ _$_$endif_$_();
   }
 };
 
-var comm = WT.initAjaxComm(sessionUrl, handleResponse);
-
 function setSessionUrl(url) {
-  sessionUrl = url;
-  comm.setUrl(url);
+  if (url.indexOf("://") != -1 || url[0] == '/')
+    sessionUrl = url;
+  else
+    sessionUrl = deployUrl + url;
+
+  if (comm)
+    comm.setUrl(url);
 }
 
+setSessionUrl(_$_SESSION_URL_$_);
+
+var comm = WT.initAjaxComm(sessionUrl, handleResponse);
+
 function doPollTimeout() {
   responsePending.abort();
   responsePending = null;
@@ -2324,7 +2420,8 @@ function scheduleUpdate() {
 
 _$_$if_WEB_SOCKETS_$_();
   if (websocket.state != WebSocketsUnavailable) {
-    if (typeof window.WebSocket === 'undefined')
+    if (typeof window.WebSocket === 'undefined'
+        && typeof window.MozWebSocket === 'undefined')
       websocket.state = WebSocketsUnavailable;
     else {
       var ws = websocket.socket;
@@ -2346,12 +2443,16 @@ _$_$if_WEB_SOCKETS_$_();
 	    var query = sessionUrl.substr(sessionUrl.indexOf('?'));
 	    wsurl = "ws" + location.protocol.substr(4)
 	      + "//" + location.hostname + ":"
-	     + location.port + _$_DEPLOY_PATH_$_ + query;
+	     + location.port + deployUrl + query;
 	  }
 
-	  // console.log("Url: " + wsurl);
+	  wsurl += "&request=ws";
+
+	  if (typeof window.WebSocket !== 'undefined')
+	    websocket.socket = ws = new WebSocket(wsurl);
+	  else
+	    websocket.socket = ws = new MozWebSocket(wsurl);
 
-	  websocket.socket = ws = new WebSocket(wsurl);
 	  if (websocket.keepAlive)
 	    clearInterval(websocket.keepAlive);
 	  websocket.keepAlive = null;
@@ -2363,19 +2464,39 @@ _$_$if_WEB_SOCKETS_$_();
 	  };
 
 	  ws.onerror = function(event) {
+	    /*
+	     * Sometimes, we can connect but cannot send data
+	     */
+	    if (reconnectTries == 3 && websocket.state == WebSocketsUnknown)
+	      websocket.state = WebSocketsUnavailable;
 	    reconnect();
 	  };
 
 	  ws.onclose = function(event) {
+	    /*
+	     * Sometimes, we can connect but cannot send data
+	     */
+	    if (websocket.reconnectTries == 3
+		&& websocket.state == WebSocketsUnknown)
+	      websocket.state = WebSocketsUnavailable;
 	    reconnect();
 	  };
 
 	  ws.onopen = function(event) {
-	    websocket.state = WebSocketsWorking;
+	    /*
+	     * WebSockets are suppossedly reliable, but there is nothing
+	     * in the protocol that makes them so...
+	     *
+	     * WebSockets are supposedly using a ping/pong protocol to
+	     * motivate proxies to keep connections open, but we've never
+	     * seen a browser pinging us ?
+	     *
+	     * So, we ping pong ourselves. It costs virtually nothing.
+	     */
 	    websocket.keepAlive = setInterval
 	    (function() {
 	       if (ws.readyState == 1)
-		 ws.send('&signal=none');
+		 ws.send('&signal=ping');
 	       else {
 		 clearInterval(websocket.keepAlive);
 		 websocket.keepAlive = null;
@@ -2416,15 +2537,14 @@ _$_$endif_$_();
   }
 }
 
-var ackUpdateId = 0;
-
-function responseReceived(updateId) {
+var ackUpdateId = _$_ACK_UPDATE_ID_$_, ackPuzzle = null;
+function responseReceived(updateId, puzzle) {
+  ackPuzzle = puzzle;
   ackUpdateId = updateId;
   comm.responseReceived(updateId);
 }
 
 var pageId = 0;
-
 function setPage(id)
 {
   pageId = id;
@@ -2471,6 +2591,24 @@ function sendUpdate() {
   }
 
   data.result += '&ackId=' + ackUpdateId + '&pageId=' + pageId;
+
+  if (ackPuzzle) {
+    var solution = '';
+    var d = $('#' + ackPuzzle).get(0);
+    if (d) {
+      d = d.parentNode;
+
+      for (; !WT.hasTag(d, 'BODY'); d = d.parentNode) {
+	if (d.id) {
+	  if (solution != '') solution += ',';
+	  solution += d.id;
+	}
+      }
+    }
+
+    data.result += '&ackPuzzle=' + encodeURIComponent(solution);
+  }
+
   var params = "_$_PARAMS_$_";
   if (params.length > 0)
     data.result += '&' + params;
@@ -2617,8 +2755,9 @@ function loadScript(uri, symbol, tries)
     };
     var h = document.getElementsByTagName('head')[0];
     h.appendChild(s);
-  } else
+  } else {
     jsLoaded(uri);
+  }
 };
 
 function ImagePreloader(uris, callback) {
